@@ -105,8 +105,9 @@ use std::time::Duration;
 use cached::proc_macro::cached;
 
 /// Use a timed cache with a TTL of 60s
+/// that refreshes the entry TTL on cache hit,
 /// and a `(String, String)` cache key
-#[cached(time=60)]
+#[cached(time=60, time_refresh=true)]
 fn keyed(a: String, b: String) -> usize {
     let size = a.len() + b.len();
     sleep(Duration::new(size as u64, 0));
@@ -171,8 +172,7 @@ use cached::Return;
 /// Get a `cached::Return` value that indicates
 /// whether the value returned came from the cache:
 /// `cached::Return.was_cached`.
-/// Use an LRU cache with a TTL of 60s
-/// and a `String` cache key.
+/// Use an LRU cache and a `String` cache key.
 #[cached(size=1, with_cached_flag = true)]
 fn calculate(a: String) -> Return<String> {
     Return::new(a)
@@ -273,6 +273,47 @@ fn keyed(a: String) -> Option<usize> {
 # pub fn main() { }
 ```
 
+```rust
+use std::thread::sleep;
+use std::time::Duration;
+use cached::proc_macro::cached;
+
+/// Use a timed cache with a TTL of 60s.
+/// Run a background thread to continuously refresh a specific key.
+#[cached(time = 60, key = "String", convert = r#"{ String::from(a) }"#)]
+fn keyed(a: &str) -> usize {
+    a.len()
+}
+pub fn main() {
+    std::thread::spawn(|| {
+        loop {
+            sleep(Duration::from_secs(60));
+            keyed_prime_cache("a");
+        }
+    });
+}
+```
+
+```rust
+use std::thread::sleep;
+use std::time::Duration;
+use cached::proc_macro::once;
+
+/// Run a background thread to continuously refresh a singleton.
+#[once]
+fn keyed() -> String {
+    // do some long http request
+    "some data".to_string()
+}
+pub fn main() {
+    std::thread::spawn(|| {
+        loop {
+            sleep(Duration::from_secs(60));
+            keyed_prime_cache();
+        }
+    });
+}
+```
 
 ----
 
