@@ -175,7 +175,7 @@ fn test_timed_sized_cache() {
 cached! {
     STRING_CACHE_EXPLICIT: SizedCache<(String, String), String> = SizedCache::with_size(1);
     fn string_1(a: String, b: String) -> String = {
-        a + &b
+        a + b.as_ref()
     }
 }
 
@@ -896,3 +896,247 @@ async fn test_cached_sync_writes_a() {
     assert_eq!(a, b.await);
     assert_eq!(a, c.await);
 }
+
+#[cached(size = 2)]
+fn cached_smartstring(s: smartstring::alias::String) -> smartstring::alias::String {
+    if s == "very stringy" {
+        smartstring::alias::String::from("equal")
+    } else {
+        smartstring::alias::String::from("not equal")
+    }
+}
+
+#[test]
+fn test_cached_smartstring() {
+    smartstring::validate();
+
+    let mut string = smartstring::alias::String::new();
+    string.push_str("very stringy");
+    assert_eq!("equal", cached_smartstring(string.clone()));
+    {
+        let cache = CACHED_SMARTSTRING.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(0));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+
+    assert_eq!("equal", cached_smartstring(string.clone()));
+    {
+        let cache = CACHED_SMARTSTRING.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(1));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+
+    let string = smartstring::alias::String::from("also stringy");
+    assert_eq!("not equal", cached_smartstring(string));
+    {
+        let cache = CACHED_SMARTSTRING.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(1));
+        assert_eq!(cache.cache_misses(), Some(2));
+    }
+}
+
+#[cached(
+    size = 2,
+    key = "smartstring::alias::String",
+    convert = r#"{ smartstring::alias::String::from(s) }"#
+)]
+fn cached_smartstring_from_str(s: &str) -> bool {
+    s == "true"
+}
+
+#[test]
+fn test_cached_smartstring_from_str() {
+    smartstring::validate();
+
+    assert!(cached_smartstring_from_str("true"));
+    {
+        let cache = CACHED_SMARTSTRING_FROM_STR.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(0));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+
+    assert!(cached_smartstring_from_str("true"));
+    {
+        let cache = CACHED_SMARTSTRING_FROM_STR.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(1));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+
+    assert!(!cached_smartstring_from_str("false"));
+    {
+        let cache = CACHED_SMARTSTRING_FROM_STR.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(1));
+        assert_eq!(cache.cache_misses(), Some(2));
+    }
+}
+
+#[cached(
+    time = 1,
+    time_refresh = true,
+    key = "String",
+    convert = r#"{ String::from(s) }"#
+)]
+fn cached_timed_refresh(s: &str) -> bool {
+    s == "true"
+}
+
+#[test]
+fn test_cached_timed_refresh() {
+    assert!(cached_timed_refresh("true"));
+    {
+        let cache = CACHED_TIMED_REFRESH.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(0));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+
+    assert!(cached_timed_refresh("true"));
+    {
+        let cache = CACHED_TIMED_REFRESH.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(1));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_refresh("true"));
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_refresh("true"));
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_refresh("true"));
+    {
+        let cache = CACHED_TIMED_REFRESH.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(4));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+}
+
+#[cached(
+    size = 2,
+    time = 1,
+    time_refresh = true,
+    key = "String",
+    convert = r#"{ String::from(s) }"#
+)]
+fn cached_timed_sized_refresh(s: &str) -> bool {
+    s == "true"
+}
+
+#[test]
+fn test_cached_timed_sized_refresh() {
+    assert!(cached_timed_sized_refresh("true"));
+    {
+        let cache = CACHED_TIMED_SIZED_REFRESH.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(0));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+
+    assert!(cached_timed_sized_refresh("true"));
+    {
+        let cache = CACHED_TIMED_SIZED_REFRESH.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(1));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_sized_refresh("true"));
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_sized_refresh("true"));
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_sized_refresh("true"));
+    {
+        let cache = CACHED_TIMED_SIZED_REFRESH.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(4));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+}
+
+#[cached(
+    size = 2,
+    time = 1,
+    time_refresh = true,
+    key = "String",
+    convert = r#"{ String::from(s) }"#
+)]
+fn cached_timed_sized_refresh_prime(s: &str) -> bool {
+    s == "true"
+}
+
+#[test]
+fn test_cached_timed_sized_refresh_prime() {
+    assert!(cached_timed_sized_refresh_prime("true"));
+    {
+        let cache = CACHED_TIMED_SIZED_REFRESH_PRIME.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(0));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+    assert!(cached_timed_sized_refresh_prime("true"));
+    {
+        let cache = CACHED_TIMED_SIZED_REFRESH_PRIME.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(1));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_sized_refresh_prime_prime_cache("true"));
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_sized_refresh_prime_prime_cache("true"));
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_sized_refresh_prime_prime_cache("true"));
+
+    // stats unchanged (other than this new hit) since we kept priming
+    assert!(cached_timed_sized_refresh_prime("true"));
+    {
+        let cache = CACHED_TIMED_SIZED_REFRESH_PRIME.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(2));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+}
+
+#[cached(size = 2, time = 1, key = "String", convert = r#"{ String::from(s) }"#)]
+fn cached_timed_sized_prime(s: &str) -> bool {
+    s == "true"
+}
+
+#[test]
+fn test_cached_timed_sized_prime() {
+    assert!(cached_timed_sized_prime("true"));
+    {
+        let cache = CACHED_TIMED_SIZED_PRIME.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(0));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+    assert!(cached_timed_sized_prime("true"));
+    {
+        let cache = CACHED_TIMED_SIZED_PRIME.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(1));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_sized_prime_prime_cache("true"));
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_sized_prime_prime_cache("true"));
+    std::thread::sleep(std::time::Duration::from_millis(500));
+    assert!(cached_timed_sized_prime_prime_cache("true"));
+
+    // stats unchanged (other than this new hit) since we kept priming
+    assert!(cached_timed_sized_prime("true"));
+    {
+        let cache = CACHED_TIMED_SIZED_PRIME.lock().unwrap();
+        assert_eq!(cache.cache_hits(), Some(2));
+        assert_eq!(cache.cache_misses(), Some(1));
+    }
+}
+
+#[once]
+fn once_for_priming() -> bool {
+    true
+}
+
+#[test]
+fn test_once_for_priming() {
+    assert!(once_for_priming_prime_cache());
+    {
+        let cache = ONCE_FOR_PRIMING.read().unwrap();
+        assert!(cache.is_some());
+    }
+}
